@@ -88,6 +88,8 @@ static void remove_expired_aliases(aliaslist_T *list, size_t index)
     __attribute__((nonnull));
 static void shift_index(aliaslist_T *list, ptrdiff_t inc)
     __attribute__((nonnull));
+static bool is_redir_fd(const wchar_t *s)
+    __attribute__((nonnull,pure));
 static bool print_alias(const wchar_t *name, const alias_T *alias, bool prefix);
 
 
@@ -175,7 +177,7 @@ void remove_all_aliases(void)
 /* Returns the value of the specified alias (or null if there is no such). */
 const wchar_t *get_alias_value(const wchar_t *aliasname)
 {
-    alias_T *alias = ht_get(&aliases, aliasname).value;
+    const alias_T *alias = ht_get(&aliases, aliasname).value;
 
     if (alias)
 	return alias->value;
@@ -296,7 +298,8 @@ substitute_alias:
     }
     /* `i' is the starting index of the alias name and `j' is the ending index*/
 
-    if (i < j && is_token_delimiter_char(buf->contents[j])) {
+    if (i < j && is_token_delimiter_char(buf->contents[j])
+	    && !is_redir_fd(buf->contents + i)) {
 	wchar_t savechar = buf->contents[j];
 	buf->contents[j] = L'\0';
 	alias_T *alias = ht_get(&aliases, buf->contents + i).value;
@@ -339,6 +342,18 @@ substitute_alias:
  *    echo a
  */
 
+/* Returns true iff the specified string starts with any number of digits
+ * followed by L'<' or L'>'. */
+/* An IO_NUMBER token, which specifies the file descriptor a redirection
+ * affects, must not be alias-substituted. This function check if the word is
+ * such a token. */
+bool is_redir_fd(const wchar_t *s)
+{
+    while (iswdigit(*s))
+	s++;
+    return *s == L'<' || *s == L'>';
+}
+
 /* Prints an alias definition to the standard output.
  * On error, an error message is printed to the standard error.
  * Returns true iff successful. */
@@ -371,7 +386,7 @@ bool print_alias(const wchar_t *name, const alias_T *alias, bool prefix)
  * Returns true iff a non-global alias had been defined and was printed. */
 bool print_alias_if_defined(const wchar_t *aliasname, bool user_friendly)
 {
-    alias_T *alias = ht_get(&aliases, aliasname).value;
+    const alias_T *alias = ht_get(&aliases, aliasname).value;
     if (alias && !(alias->flags & AF_GLOBAL)) {
 	if (!user_friendly) {
 	    return print_alias(aliasname, alias, true);
@@ -454,8 +469,8 @@ int alias_builtin(int argc, void **argv)
 		    xerror(0, Ngt("`%ls': alias cannot contain newlines"), arg);
 	    } else if (nameend != arg && *nameend == L'\0') {
 		/* print alias */
-		alias_T *alias = ht_get(&aliases, arg).value;
-		if (alias) {
+		const alias_T *alias = ht_get(&aliases, arg).value;
+		if (alias != NULL) {
 		    if (!print_alias(arg, alias, prefix))
 			break;
 		} else {
