@@ -62,12 +62,15 @@ static char *get_process_status_string(const process_T *p, bool *needfree)
     __attribute__((nonnull,malloc,warn_unused_result));
 static char *get_job_status_string(const job_T *job, bool *needfree)
     __attribute__((nonnull,malloc,warn_unused_result));
+static int print_job_status(
+	size_t jobnumber, _Bool changedonly, _Bool verbose, FILE *f)
+    __attribute__((nonnull));
 static size_t get_jobnumber_from_name(const wchar_t *name)
     __attribute__((nonnull,pure));
 static size_t get_jobnumber_from_pid(pid_t pid)
     __attribute__((pure));
 
-static void jobs_builtin_print_job(size_t jobnumber,
+static bool jobs_builtin_print_job(size_t jobnumber,
 	bool verbose, bool changedonly, bool pgidonly,
 	bool runningonly, bool stoppedonly);
 static int continue_job(size_t jobnumber, job_T *job, bool fg)
@@ -631,7 +634,7 @@ char *get_process_status_string(const process_T *p, bool *needfree)
 	return (char *) gt("Running");
     case JS_STOPPED:
 	*needfree = true;
-	return malloc_printf(gt("Stopped(SIG%s)"),
+	return malloc_printf(gt("Stopped(SIG%ls)"),
 		get_signal_name(WSTOPSIG(p->pr_statuscode)));
     case JS_DONE:
 	status = p->pr_statuscode;
@@ -653,11 +656,10 @@ exitstatus:
 	    sig = WTERMSIG(status);
 #ifdef WCOREDUMP
 	    if (WCOREDUMP(sig))
-		return malloc_printf(gt("Killed (SIG%s: core dumped)"),
+		return malloc_printf(gt("Killed (SIG%ls: core dumped)"),
 			get_signal_name(sig));
 #endif
-	    return malloc_printf(gt("Killed (SIG%s)"),
-		    get_signal_name(sig));
+	    return malloc_printf(gt("Killed (SIG%ls)"), get_signal_name(sig));
 	}
     }
     assert(false);
@@ -692,12 +694,15 @@ char *get_job_status_string(const job_T *job, bool *needfree)
  * If `changedonly' is true, the job is printed only if the `j_statuschanged'
  * flag is true.
  * If `verbose' is true, the status is printed in the process-wise format rather
- * than the usual job-wise format. */
-void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
+ * than the usual job-wise format.
+ * Returns zero if successful. Returns errno if `fprintf' failed. */
+int print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 {
+    int result = 0;
+
     job_T *job = get_job(jobnumber);
     if (!job || (changedonly && !job->j_statuschanged) || job->j_nonotify)
-	return;
+	return result;
 
     char current;
     if      (jobnumber == current_jobnumber)  current = '+';
@@ -712,8 +717,9 @@ void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 	/* TRANSLATORS: the translated format string can be different 
 	 * from the original only in the number of spaces. This is required
 	 * for POSIX compliance. */
-	fprintf(f, gt("[%zu] %c %-20s %ls\n"),
+	result = fprintf(f, gt("[%zu] %c %-20s %ls\n"),
 		jobnumber, current, status, jobname);
+	result = (result >= 0) ? 0 : errno;
 
 	if (needfree)
 	    free(status);
@@ -730,12 +736,13 @@ void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 	/* TRANSLATORS: the translated format string can be different 
 	 * from the original only in the number of spaces. This is required
 	 * for POSIX compliance. */
-	fprintf(f, gt("[%zu] %c %5jd %-20s   %ls\n"),
+	result = fprintf(f, gt("[%zu] %c %5jd %-20s   %ls\n"),
 		jobnumber, current, (intmax_t) pid, status, jobname);
+	result = (result >= 0) ? 0 : errno;
 	if (needfree)
 	    free(status);
 
-	for (size_t i = 1; i < job->j_pcount; i++) {
+	for (size_t i = 1; result == 0 && i < job->j_pcount; i++) {
 	    pid = job->j_procs[i].pr_pid;
 	    status = get_process_status_string(&job->j_procs[i], &needfree);
 	    jobname = job->j_procs[i].pr_name;
@@ -743,8 +750,9 @@ void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 	    /* TRANSLATORS: the translated format string can be different 
 	     * from the original only in the number of spaces. This is required
 	     * for POSIX compliance. */
-	    fprintf(f, gt("      %5jd %-20s | %ls\n"),
+	    result = fprintf(f, gt("      %5jd %-20s | %ls\n"),
 		    (intmax_t) pid, (posixly_correct ? "" : status), jobname);
+	    result = (result >= 0) ? 0 : errno;
 	    if (needfree)
 		free(status);
 	}
@@ -752,6 +760,8 @@ void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
     job->j_statuschanged = false;
     if (job->j_status == JS_DONE)
 	remove_job(jobnumber);
+
+    return result;
 }
 
 /* Prints the status of jobs which have been changed but not reported. */
@@ -790,10 +800,10 @@ void notify_signaled_job(size_t jobnumber)
 	    break;
 	default:
 #if HAVE_STRSIGNAL
-	    fprintf(stderr, gt("Process killed by SIG%s: %s\n"),
+	    fprintf(stderr, gt("Process killed by SIG%ls: %s\n"),
 		    get_signal_name(sig), strsignal(sig));
 #else
-	    fprintf(stderr, gt("Process killed by SIG%s\n"),
+	    fprintf(stderr, gt("Process killed by SIG%ls\n"),
 		    get_signal_name(sig));
 #endif
 	    break;
@@ -894,7 +904,6 @@ int jobs_builtin(int argc, void **argv)
 
     bool verbose = false, changedonly = false, pgidonly = false;
     bool runningonly = false, stoppedonly = false;
-    bool err = false;
     wchar_t opt;
 
     xoptind = 0, xopterr = true;
@@ -922,7 +931,6 @@ int jobs_builtin(int argc, void **argv)
     nextforceexit = true;
     apply_curstop();
 
-    clearerr(stdout);
     if (xoptind < argc) {
 	/* print the specified jobs */
 	do {
@@ -931,58 +939,62 @@ int jobs_builtin(int argc, void **argv)
 		jobspec++;
 	    } else if (posixly_correct) {
 		xerror(0, Ngt("%ls: invalid job specification"), ARGV(xoptind));
-		err = true;
 		continue;
 	    }
 	    size_t jobnumber = get_jobnumber_from_name(jobspec);
 	    if (jobnumber >= joblist.length) {
 		xerror(0, Ngt("%ls: ambiguous job specification"),
 			ARGV(xoptind));
-		err = true;
 	    } else if (jobnumber == 0 || joblist.contents[jobnumber] == NULL) {
 		xerror(0, Ngt("%ls: no such job"), ARGV(xoptind));
-		err = true;
 	    } else {
-		jobs_builtin_print_job(jobnumber, verbose, changedonly,
-			pgidonly, runningonly, stoppedonly);
+		if (!jobs_builtin_print_job(jobnumber, verbose,
+			changedonly, pgidonly, runningonly, stoppedonly))
+		    return Exit_FAILURE;
 	    }
 	} while (++xoptind < argc);
     } else {
 	/* print all jobs */
 	for (size_t i = 1; i < joblist.length; i++) {
-	    jobs_builtin_print_job(i, verbose, changedonly, pgidonly,
-		    runningonly, stoppedonly);
+	    if (!jobs_builtin_print_job(i, verbose, changedonly, pgidonly,
+		    runningonly, stoppedonly))
+		return Exit_FAILURE;
 	}
     }
 
-    if (!ferror(stdout)) {
-	return err ? Exit_FAILURE : Exit_SUCCESS;
-    } else {
-	xerror(0, Ngt("cannot print to standard output"));
-	return Exit_FAILURE;
-    }
+    return (yash_error_message_count == 0) ? Exit_SUCCESS : Exit_FAILURE;
 }
 
-/* Prints the job status */
-void jobs_builtin_print_job(size_t jobnumber,
+/* Prints the job status.
+ * On an I/O error, an error message is printed to the standard error and false
+ * is returned. */
+bool jobs_builtin_print_job(size_t jobnumber,
 	bool verbose, bool changedonly, bool pgidonly,
 	bool runningonly, bool stoppedonly)
 {
     job_T *job = get_job(jobnumber);
 
     if (job == NULL)
-	return;
+	return true;
     if (runningonly && job->j_status != JS_RUNNING)
-	return;
+	return true;
     if (stoppedonly && job->j_status != JS_STOPPED)
-	return;
+	return true;
 
+    int err;
     if (pgidonly) {
 	if (changedonly && !job->j_statuschanged)
-	    return;
-	printf("%jd\n", imaxabs(job->j_pgid));
+	    return true;
+	int result = printf("%jd\n", imaxabs(job->j_pgid));
+	err = (result >= 0) ? 0 : errno;
     } else {
-	print_job_status(jobnumber, changedonly, verbose, stdout);
+	err = print_job_status(jobnumber, changedonly, verbose, stdout);
+    }
+    if (err != 0) {
+	xerror(err, Ngt("cannot print to standard output"));
+	return false;
+    } else {
+	return true;
     }
 }
 
@@ -1012,7 +1024,6 @@ const char jobs_help[] = Ngt(
 int fg_builtin(int argc, void **argv)
 {
     bool fg = wcscmp(argv[0], L"fg") == 0;
-    bool err = false;
     wchar_t opt;
 
     xoptind = 0, xopterr = true;
@@ -1048,22 +1059,18 @@ int fg_builtin(int argc, void **argv)
 		jobspec++;
 	    } else if (posixly_correct) {
 		xerror(0, Ngt("%ls: invalid job specification"), ARGV(xoptind));
-		err = true;
 		continue;
 	    }
 	    size_t jobnumber = get_jobnumber_from_name(jobspec);
 	    if (jobnumber >= joblist.length) {
 		xerror(0, Ngt("%ls: ambiguous job specification"),
 			ARGV(xoptind));
-		err = true;
 	    } else if (jobnumber == 0
 		    || (job = joblist.contents[jobnumber]) == NULL
 		    || job->j_pgid < 0) {
 		xerror(0, Ngt("%ls: no such job"), ARGV(xoptind));
-		err = true;
 	    } else if (job->j_pgid == 0) {
 		xerror(0, Ngt("%ls: not job-controlled job"), ARGV(xoptind));
-		err = true;
 	    } else {
 		status = continue_job(jobnumber, job, fg);
 	    }
@@ -1072,13 +1079,16 @@ int fg_builtin(int argc, void **argv)
 	if (current_jobnumber == 0 ||
 		(job = joblist.contents[current_jobnumber])->j_pgid <= 0) {
 	    xerror(0, Ngt("no current job"));
-	    err = true;
 	} else {
 	    status = continue_job(current_jobnumber, job, fg);
 	}
     }
 
-    return (status != 0) ? status : err ? Exit_FAILURE : Exit_SUCCESS;
+    if (status != 0)
+	return status;
+    if (yash_error_message_count != 0)
+	return Exit_FAILURE;
+    return Exit_SUCCESS;
 }
 
 /* Continues execution of the specified job.
@@ -1173,7 +1183,6 @@ const char bg_help[] = Ngt(
 int wait_builtin(int argc, void **argv)
 {
     bool jobcontrol = doing_job_control_now;
-    bool err = false;
     int status = Exit_SUCCESS;
     wchar_t opt;
 
@@ -1202,7 +1211,6 @@ int wait_builtin(int argc, void **argv)
 		long pid;
 		if (!xwcstol(jobspec, 10, &pid) || pid < 0) {
 		    xerror(0, Ngt("%ls: invalid job specification"), jobspec);
-		    err = true;
 		    continue;
 		}
 		jobnumber = get_jobnumber_from_pid((pid_t) pid);
@@ -1211,14 +1219,13 @@ int wait_builtin(int argc, void **argv)
 	    if (jobnumber >= joblist.length) {
 		xerror(0, Ngt("%ls: ambiguous job specification"),
 			ARGV(xoptind));
-		err = true;
 	    } else if (jobnumber == 0
 		    || (job = joblist.contents[jobnumber]) == NULL
 		    || job->j_pgid < 0) {
 		status = Exit_NOTFOUND;
 	    } else {
 		status = wait_for_job(jobnumber, jobcontrol, jobcontrol, true);
-		if (!status) {
+		if (status == 0) {
 		    status = calc_status_of_job(job);
 		} else {
 		    assert(TERMSIGOFFSET >= 128);
@@ -1245,7 +1252,11 @@ int wait_builtin(int argc, void **argv)
 	}
     }
 
-    return status ? status : err ? Exit_FAILURE : Exit_SUCCESS;
+    if (status != 0)
+	return status;
+    if (yash_error_message_count != 0)
+	return Exit_FAILURE;
+    return Exit_SUCCESS;
 }
 
 /* Checks if the shell has any job to wait for. */
@@ -1253,7 +1264,8 @@ bool wait_has_job(bool jobcontrol)
 {
     /* print/remove already-finished jobs */
     if (jobcontrol && is_interactive_now && !posixly_correct) {
-	print_job_status_all();
+	for (size_t i = 1; i < joblist.length; i++)
+	    print_job_status(i, true, false, stdout);
     } else {
 	for (size_t i = 1; i < joblist.length; i++) {
 	    job_T *job = joblist.contents[i];
@@ -1285,7 +1297,6 @@ const char wait_help[] = Ngt(
 int disown_builtin(int argc, void **argv)
 {
     bool all = false;
-    bool err = false;
     wchar_t opt;
 
     xoptind = 0, xopterr = true;
@@ -1311,31 +1322,26 @@ int disown_builtin(int argc, void **argv)
 		jobspec++;
 	    } else if (posixly_correct) {
 		xerror(0, Ngt("%ls: invalid job specification"), ARGV(xoptind));
-		err = true;
 		continue;
 	    }
 	    size_t jobnumber = get_jobnumber_from_name(jobspec);
 	    if (jobnumber >= joblist.length) {
 		xerror(0, Ngt("%ls: ambiguous job specification"),
 			ARGV(xoptind));
-		err = true;
 	    } else if (jobnumber == 0 || joblist.contents[jobnumber] == NULL) {
 		xerror(0, Ngt("%ls: no such job"), ARGV(xoptind));
-		err = true;
 	    } else {
 		remove_job(jobnumber);
 	    }
 	} while (++xoptind < argc);
     } else {
-	if (current_jobnumber == 0 || get_job(current_jobnumber) == NULL) {
+	if (current_jobnumber == 0 || get_job(current_jobnumber) == NULL)
 	    xerror(0, Ngt("no current job"));
-	    err = true;
-	} else {
+	else
 	    remove_job(current_jobnumber);
-	}
     }
 
-    return err ? Exit_FAILURE : Exit_SUCCESS;
+    return (yash_error_message_count == 0) ? Exit_SUCCESS : Exit_FAILURE;
 }
 
 #if YASH_ENABLE_HELP
