@@ -61,7 +61,11 @@
  * is no longer sticking. */
 
 
-#if !HAVE_WCWIDTH
+#if HAVE_WCWIDTH
+# ifndef wcwidth
+extern int wcwidth(wchar_t c);
+# endif
+#else
 # undef wcwidth
 # define wcwidth(c) (iswprint(c) ? 1 : 0)
 #endif
@@ -355,13 +359,17 @@ void le_display_finalize(void)
     clean_up();
 }
 
-/* Clears prompt, edit line and info area on the screen. */
-void le_display_clear(void)
+/* Clears prompt, edit line and info area on the screen.
+ * If `clear' is true, also prints the "clear" capability. */
+void le_display_clear(bool clear)
 {
     if (display_active) {
 	lebuf_init(currentp);
 	lebuf_print_sgr0();
-	go_to((le_pos_T) { 0, 0 });
+	if (clear)
+	    lebuf_print_clear();
+	else
+	    go_to((le_pos_T) { 0, 0 });
 	clean_up();
     }
 }
@@ -501,8 +509,10 @@ void le_display_update(bool cursor)
 
     /* set cursor position */
     assert(le_main_index <= le_main_buffer.length);
-    if (cursor)
+    if (cursor) {
 	go_to_index(le_main_index);
+	update_styler();
+    }
 }
 
 /* Prints a dummy string that moves the cursor to the first column of the next
