@@ -863,12 +863,10 @@ void cmd_clear_and_redraw_all(wchar_t c __attribute__((unused)))
 
 void redraw_all(bool clear)
 {
-    le_display_clear();
+    le_display_clear(clear);
     le_restore_terminal();
     le_setupterm(false);
     le_set_terminal();
-    if (clear)
-	lebuf_print_clear();
 }
 
 
@@ -2168,7 +2166,7 @@ void cmd_vi_append(wchar_t c __attribute__((unused)))
 
 /* Moves the cursor to the end of the line and sets the editing mode to "vi
  * insert".*/
-void cmd_vi_append_end(wchar_t c __attribute__((unused)))
+void cmd_vi_append_to_eol(wchar_t c __attribute__((unused)))
 {
     ALERT_AND_RETURN_IF_PENDING;
 
@@ -2252,7 +2250,7 @@ void cmd_vi_change_to_eol(wchar_t c __attribute__((unused)))
 
 /* Deletes all the content of the edit line and sets the editing mode to "vi
  * insert". */
-void cmd_vi_change_all(wchar_t c __attribute__((unused)))
+void cmd_vi_change_line(wchar_t c __attribute__((unused)))
 {
     exec_motion_expect_command_line(MEC_CHANGE);
 }
@@ -2275,7 +2273,7 @@ void cmd_vi_yank_and_change_to_eol(wchar_t c __attribute__((unused)))
 
 /* Deletes all the content of the edit line, put it in the kill ring, and sets
  * the editing mode to "vi insert". */
-void cmd_vi_yank_and_change_all(wchar_t c __attribute__((unused)))
+void cmd_vi_yank_and_change_line(wchar_t c __attribute__((unused)))
 {
     exec_motion_expect_command_line(MEC_COPYCHANGE);
 }
