@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* parser.c: syntax parser */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -106,7 +106,7 @@ void comsfree(command_T *c)
 		caseitemsfree(c->c_casitems);
 		break;
 	    case CT_FUNCDEF:
-		free(c->c_funcname);
+		wordfree(c->c_funcname);
 		comsfree(c->c_funcbody);
 		break;
 	}
@@ -403,7 +403,7 @@ static wordunit_T *parse_special_word_unit(void)
     __attribute__((malloc,warn_unused_result));
 static wordunit_T *tryparse_paramexp_raw(void)
     __attribute__((malloc,warn_unused_result));
-static wordunit_T *parse_paramexp_in_brase(void)
+static wordunit_T *parse_paramexp_in_brace(void)
     __attribute__((malloc,warn_unused_result));
 static wordunit_T *parse_cmdsubst_in_paren(void)
     __attribute__((malloc,warn_unused_result));
@@ -435,6 +435,8 @@ static caseitem_T *parse_case_list(void)
     __attribute__((malloc,warn_unused_result));
 static void **parse_case_patterns(void)
     __attribute__((malloc,warn_unused_result));
+static command_T *parse_function(void)
+    __attribute__((malloc,warn_unused_result));
 static command_T *tryparse_function(void)
     __attribute__((malloc,warn_unused_result));
 static void read_heredoc_contents(redir_T *redir);
@@ -788,8 +790,8 @@ bool is_closing_brace(wchar_t c)
  * `ensure_buffer(wcslen(t))' must be done beforehand. */
 bool is_token_at(const wchar_t *t, size_t index)
 {
-    wchar_t *c = matchwcsprefix(cbuf.contents + index, t);
-    return c && is_token_delimiter_char(*c);
+    const wchar_t *c = matchwcsprefix(cbuf.contents + index, t);
+    return c != NULL && is_token_delimiter_char(*c);
 }
 
 /* Checks if there is an 'opening' token such as "(", "{" and "if" at the
@@ -797,7 +799,7 @@ bool is_token_at(const wchar_t *t, size_t index)
  * Otherwise NULL is returned. */
 const wchar_t *check_opening_token(void)
 {
-    ensure_buffer(6);
+    ensure_buffer(9);
     if (cbuf.contents[cindex] == L'(') return L"(";
     if (is_token_at(L"{",     cindex)) return L"{";
     if (is_token_at(L"if",    cindex)) return L"if";
@@ -805,6 +807,7 @@ const wchar_t *check_opening_token(void)
     if (is_token_at(L"while", cindex)) return L"while";
     if (is_token_at(L"until", cindex)) return L"until";
     if (is_token_at(L"case",  cindex)) return L"case";
+    if (is_token_at(L"function", cindex)) return L"function";
     return NULL;
 }
 
@@ -1357,14 +1360,14 @@ reparse:
     if (result->rd_type != RT_HERE && result->rd_type != RT_HERERT) {
 	result->rd_filename = parse_word(globalonly);
 	if (!result->rd_filename) {
-	    serror(Ngt("redirect target not specified"));
+	    serror(Ngt("redirection target missing"));
 	    free(result);
 	    return NULL;
 	}
     } else {
 	wchar_t *endofheredoc = parse_word_as_wcs();
 	if (endofheredoc[0] == L'\0') {
-	    serror(Ngt("here-document delimiter not specified"));
+	    serror(Ngt("here-document delimiter missing"));
 	    free(result);
 	    return NULL;
 	}
@@ -1504,7 +1507,7 @@ void skip_to_next_single_quote(void)
 	    return;
 	case L'\0':
 	    if (read_more_input() != 0) {
-		serror(Ngt("single quote not closed"));
+		serror(Ngt("single-quote not closed"));
 		return;
 	    }
 	    continue;
@@ -1530,7 +1533,7 @@ wordunit_T *parse_special_word_unit(void)
 	ensure_buffer(2);
 	switch (cbuf.contents[cindex]) {
 	case L'{':
-	    return parse_paramexp_in_brase();
+	    return parse_paramexp_in_brace();
 	case L'(':
 	    if (cbuf.contents[cindex + 1] == L'(') {
 		wordunit_T *wu = tryparse_arith();
@@ -1592,7 +1595,7 @@ fail:
 
 /* Parses a parameter expansion starting with "${".
  * `cindex' points to '{' when the function is called, and '}' when returns. */
-wordunit_T *parse_paramexp_in_brase(void)
+wordunit_T *parse_paramexp_in_brace(void)
 {
     paramexp_T *pe = xmalloc(sizeof *pe);
     pe->pe_type = 0;
@@ -1624,7 +1627,7 @@ wordunit_T *parse_paramexp_in_brase(void)
     /* ensure_buffer(2); */
     if (!posixly_correct && cbuf.contents[cindex] == L'{') {
 	pe->pe_type |= PT_NEST;
-	pe->pe_nest = parse_paramexp_in_brase();
+	pe->pe_nest = parse_paramexp_in_brace();
     } else if (!posixly_correct
 	    && (cbuf.contents[cindex] == L'`'
 		|| (cbuf.contents[cindex] == L'$'
@@ -1650,12 +1653,11 @@ parse_name:;
 	    cindex++;
 	if (namestartindex == cindex) {
 	    serror(Ngt("parameter name missing or invalid"));
-	    goto fail;
+	    goto end;
 	}
 make_name:
 	pe->pe_name = xwcsndup(
 		cbuf.contents + namestartindex, cindex - namestartindex);
-	assert(pe->pe_name != NULL);
     }
 
     /* parse indices */
@@ -1693,19 +1695,16 @@ make_name:
     case L'#':   pe->pe_type |= PT_MATCH | PT_MATCHHEAD;     goto parse_match;
     case L'%':   pe->pe_type |= PT_MATCH | PT_MATCHTAIL;     goto parse_match;
     case L'/':   pe->pe_type |= PT_SUBST | PT_MATCHLONGEST;  goto parse_match;
-    case L'}':
+    case L'}':  case L'\0':  case L'\n':
 	pe->pe_type |= PT_NONE;
 	if (pe->pe_type & PT_COLON)
 	    serror(Ngt("invalid use of `%lc' in parameter expansion"),
 		    (wint_t) L':');
-	goto check_closing_paren_and_finish;
-    case L'\0':  case L'\n':
-	serror(Ngt("`%ls' missing"), L"}");
-	goto fail;
+	goto check_closing_brace;
     default:
 	serror(Ngt("invalid character `%lc' in parameter expansion"),
 		(wint_t) cbuf.contents[cindex]);
-	goto fail;
+	goto end;
     }
 
 parse_match:
@@ -1737,19 +1736,19 @@ parse_match:
     }
     if ((pe->pe_type & PT_MASK) == PT_MATCH) {
 	pe->pe_match = parse_word_to(is_closing_brace);
-	goto check_closing_paren_and_finish;
+	goto check_closing_brace;
     } else {
 	pe->pe_match = parse_word_to(is_slash_or_closing_brace);
+	ensure_buffer(1);
+	if (cbuf.contents[cindex] != L'/')
+	    goto check_closing_brace;
     }
 
-    ensure_buffer(1);
-    if (cbuf.contents[cindex] != L'/')
-	goto check_closing_paren_and_finish;
 parse_subst:
     cindex++;
     pe->pe_subst = parse_word_to(is_closing_brace);
 
-check_closing_paren_and_finish:
+check_closing_brace:
     ensure_buffer(1);
     if (cbuf.contents[cindex] == L'}')
 	cindex++;
@@ -1759,15 +1758,12 @@ check_closing_paren_and_finish:
 	serror(Ngt("invalid use of `%lc' in parameter expansion"),
 		(wint_t) L'#');
 
+end:;
     wordunit_T *result = xmalloc(sizeof *result);
     result->next = NULL;
     result->wu_type = WT_PARAM;
     result->wu_param = pe;
     return result;
-
-fail:
-    paramfree(pe);
-    return NULL;
 }
 
 /* Parses a command substitution starting with "$(".
@@ -1991,7 +1987,16 @@ command_T *parse_compound_command(const wchar_t *command)
 	result = parse_if();
 	break;
     case L'f':
-	result = parse_for();
+	switch (command[1]) {
+	    case L'o':
+		result = parse_for();
+		break;
+	    case L'u':
+		result = parse_function();
+		break;
+	    default:
+		assert(false);
+	}
 	break;
     case L'w':
 	result = parse_while(true);
@@ -2014,16 +2019,19 @@ command_T *parse_compound_command(const wchar_t *command)
  * `type' must be either CT_GROUP or CT_SUBSHELL. */
 command_T *parse_group(commandtype_T type)
 {
-    const wchar_t *terminator;
+    const wchar_t *start, *end;
 
-    if (type == CT_GROUP) {
-	assert(cbuf.contents[cindex] == L'{');
-	terminator = L"}";
-    } else if (type == CT_SUBSHELL) {
-	assert(cbuf.contents[cindex] == L'(');
-	terminator = L")";
-    } else {
-	assert(false);
+    switch (type) {
+	case CT_GROUP:
+	    start = L"{", end = L"}";
+	    assert(is_token_at(start, cindex));
+	    break;
+	case CT_SUBSHELL:
+	    start = L"(", end = L")";
+	    assert(cbuf.contents[cindex] == start[0]);
+	    break;
+	default:
+	    assert(false);
     }
     cindex++;
 
@@ -2035,11 +2043,11 @@ command_T *parse_group(commandtype_T type)
     result->c_redirs = NULL;
     result->c_subcmds = parse_compound_list();
     if (posixly_correct && !result->c_subcmds)
-	serror(Ngt("no commands in command group"));
-    if (cbuf.contents[cindex] == terminator[0])
+	serror(Ngt("no commands between `%ls' and `%ls'"), start, end);
+    if (cbuf.contents[cindex] == end[0])
 	cindex++;
     else
-	print_errmsg_token_missing(terminator);
+	print_errmsg_token_missing(end);
     return result;
 }
 
@@ -2067,7 +2075,7 @@ command_T *parse_if(void)
 	    ic->ic_condition = parse_compound_list();
 	    if (posixly_correct && !ic->ic_condition)
 		serror(Ngt("no commands between `%ls' and `%ls'"),
-			L"if", L"then");
+			(first->next == NULL) ? L"if" : L"elif", L"then");
 	    ensure_buffer(5);
 	    if (is_token_at(L"then", cindex))
 		cindex += 4;
@@ -2152,7 +2160,8 @@ command_T *parse_for(void)
     if (is_token_at(L"do", cindex))
 	cindex += 2;
     else
-	print_errmsg_token_missing(L"do");
+	serror(Ngt("`%ls' missing"), L"do");
+	// print_errmsg_token_missing(L"do");
     result->c_forcmds = parse_compound_list();
     if (posixly_correct && !result->c_forcmds)
 	serror(Ngt("no commands between `%ls' and `%ls'"), L"do", L"done");
@@ -2214,11 +2223,14 @@ command_T *parse_case(void)
 	serror(Ngt("no word after `%ls'"), L"case");
     skip_to_next_token();
     ensure_buffer(3);
-    if (is_token_at(L"in", cindex))
+    if (is_token_at(L"in", cindex)) {
 	cindex += 2;
-    else
-	print_errmsg_token_missing(L"in");
-    result->c_casitems = parse_case_list();
+	result->c_casitems = parse_case_list();
+    } else {
+	serror(Ngt("`%ls' missing"), L"in");
+	// print_errmsg_token_missing(L"in");
+	result->c_casitems = NULL;
+    }
     ensure_buffer(5);
     if (is_token_at(L"esac", cindex))
 	cindex += 4;
@@ -2233,7 +2245,7 @@ caseitem_T *parse_case_list(void)
 {
     caseitem_T *first = NULL, **lastp = &first;
 
-    while (!cerror) {
+    do {
 	skip_to_next_token();
 	ensure_buffer(5);
 	if (is_token_at(L"esac", cindex))
@@ -2252,26 +2264,33 @@ caseitem_T *parse_case_list(void)
 	} else {
 	    break;
 	}
-    }
+    } while (!cerror);
     return first;
 }
 
 /* Parses patterns of a case item.
  * `cindex' is advanced to the next character after ')', not the next token.
- * Call `skip_blanks_and_comment' and `ensure_buffer(1)' beforehand. */
+ * Call `skip_to_next_token' and `ensure_buffer(1)' beforehand. */
 void **parse_case_patterns(void)
 {
     plist_T wordlist;
-
     pl_init(&wordlist);
+
     if (cbuf.contents[cindex] == L'(') {  /* ignore the first '(' */
 	cindex++;
 	skip_blanks_and_comment();
     }
-    while (!cerror) {
-	if (is_token_delimiter_char(cbuf.contents[cindex]))
-	    serror(Ngt("invalid character `%lc' in case pattern"),
-		    (wint_t) cbuf.contents[cindex]);
+    do {
+	if (is_token_delimiter_char(cbuf.contents[cindex])) {
+	    if (cbuf.contents[cindex] != L'\0') {
+		if (cbuf.contents[cindex] == L'\n')
+		    serror(Ngt("no word after `%ls'"), L"(");
+		else
+		    serror(Ngt("invalid character `%lc' in case pattern"),
+			    (wint_t) cbuf.contents[cindex]);
+	    }
+	    break;
+	}
 	pl_add(&wordlist, parse_word(globalonly));
 	skip_blanks_and_comment();
 	ensure_buffer(1);
@@ -2285,10 +2304,55 @@ void **parse_case_patterns(void)
 	    break;
 	}
 	skip_blanks_and_comment();
-    }
+    } while (!cerror);
     return pl_toary(&wordlist);
 }
 
+/* Parses a function definition that starts with the "function" keyword. */
+command_T *parse_function(void)
+{
+    if (posixly_correct)
+	serror(Ngt("invalid keyword `%ls'"), L"function");
+
+    assert(is_token_at(L"function", cindex));
+    cindex += 8;
+    skip_blanks_and_comment();
+
+    command_T *result = xmalloc(sizeof *result);
+    result->next = NULL;
+    result->refcount = 1;
+    result->c_type = CT_FUNCDEF;
+    result->c_lineno = cinfo->lineno;
+    result->c_redirs = NULL;
+    result->c_funcname = parse_word(globalonly);
+    if (result->c_funcname == NULL)
+	serror(Ngt("no word after `%ls'"), L"function");
+    skip_blanks_and_comment();
+
+    /* parse parentheses */
+    size_t savecindex = cindex;
+    if (cbuf.contents[cindex] == L'(') {
+	cindex++;
+	skip_blanks_and_comment();
+	if (cbuf.contents[cindex] == L')')
+	    cindex++;
+	else
+	    cindex = savecindex;
+    }
+    skip_to_next_token();
+
+    /* parse function body */
+    const wchar_t *t = check_opening_token();
+    if (t == NULL) {
+	serror(Ngt("function body must be compound command"));
+	result->c_funcbody = NULL;
+    } else {
+	result->c_funcbody = parse_compound_command(t);
+    }
+
+    return result;
+}
+
 /* Parses a function definition if any.
  * This function may do line continuations, increasing the line number. */
 command_T *tryparse_function(void)
@@ -2330,7 +2394,11 @@ command_T *tryparse_function(void)
     result->c_type = CT_FUNCDEF;
     result->c_lineno = lineno;
     result->c_redirs = NULL;
-    result->c_funcname = xwcsndup(cbuf.contents + savecindex, namelen);
+    result->c_funcname = xmalloc(sizeof *result->c_funcname);
+    result->c_funcname->next = NULL;
+    result->c_funcname->wu_type = WT_STRING;
+    result->c_funcname->wu_string =
+	xwcsndup(cbuf.contents + savecindex, namelen);
     result->c_funcbody = body;
     return result;
 
@@ -2537,27 +2605,44 @@ bool parse_string(parseinfo_T *restrict info, wordunit_T **restrict result)
 const char *get_errmsg_unexpected_token(const wchar_t *t)
 {
     switch (t[0]) {
-	case L')': return Ngt("`%ls' without matching `('");
-	case L'}': return Ngt("`%ls' without matching `{'");
-	case L';': return Ngt("`%ls' used outside `case'");
-	case L'!': return Ngt("`%ls' cannot be used as command name");
-	case L'i': return Ngt("`%ls' cannot be used as command name");
-	case L'f': return Ngt("`%ls' without matching `if'");
-	case L't': return Ngt("`%ls' used without `if'");
+	case L')':
+	    assert(wcscmp(t, L")") == 0);
+	    return Ngt("`%ls' without matching `('");
+	case L'}':
+	    assert(wcscmp(t, L"}") == 0);
+	    return Ngt("`%ls' without matching `{'");
+	case L';':
+	    assert(wcscmp(t, L";;") == 0);
+	    return Ngt("`%ls' used outside `case'");
+	case L'!':
+	    assert(wcscmp(t, L"!") == 0);
+	    return Ngt("`%ls' cannot be used as command name");
+	case L'i':
+	    assert(wcscmp(t, L"in") == 0);
+	    return Ngt("`%ls' cannot be used as command name");
+	case L'f':
+	    assert(wcscmp(t, L"fi") == 0);
+	    return Ngt("`%ls' without matching `if' and/or `then'");
+	case L't':
+	    assert(wcscmp(t, L"then") == 0);
+	    return Ngt("`%ls' used without `if' or `elif'");
 	case L'd':
 	    assert(t[1] == L'o');
-	    if (t[2] == L'\0')
+	    if (t[2] == L'\0') {
+		assert(wcscmp(t, L"do") == 0);
 		return Ngt("`%ls' used without `for', `while', or `until'");
-	    else
+	    } else {
+		assert(wcscmp(t, L"done") == 0);
 		return Ngt("`%ls' without matching `do'");
+	    }
 	case L'e':
-	    if (t[1] == L's')
+	    if (t[1] == L's') {
+		assert(wcscmp(t, L"esac") == 0);
 		return Ngt("`%ls' without matching `case'");
-	    else
-		if (t[2] == L's')
-		    return Ngt("`%ls' used without `if'");
-		else
-		    return Ngt("`%ls' used without `if'");
+	    } else {
+		assert(wcscmp(t, L"else") == 0 || wcscmp(t, L"elif") == 0);
+		return Ngt("`%ls' used without `if' and/or `then'");
+	    }
 	default:
 	    assert(false);
     }
@@ -2566,10 +2651,12 @@ const char *get_errmsg_unexpected_token(const wchar_t *t)
 void print_errmsg_token_missing(const wchar_t *t)
 {
     const wchar_t *atoken = check_closing_token();
-    if (atoken)
+    if (atoken) {
 	serror(get_errmsg_unexpected_token(atoken), atoken);
-    else
+	serror(Ngt("(maybe you missed `%ls'?)"), t);
+    } else {
 	serror(Ngt("`%ls' missing"), t);
+    }
 }
 
 
@@ -2950,7 +3037,11 @@ void print_function_definition(
 {
     assert(c->c_type == CT_FUNCDEF);
 
-    wb_cat(&pr->buffer, c->c_funcname);
+    if (c->c_funcname->next != NULL
+	    || c->c_funcname->wu_type != WT_STRING
+	    || !is_name(c->c_funcname->wu_string))
+	wb_cat(&pr->buffer, L"function ");
+    print_word(pr, c->c_funcname, indent);
     wb_cat(&pr->buffer, L"()");
     print_space_or_newline(pr);
 
