@@ -60,7 +60,6 @@ extern char **environ;
 #define VAR_PROMPT_COMMAND            "PROMPT_COMMAND"
 #define VAR_PS1                       "PS1"
 #define VAR_PS2                       "PS2"
-#define VAR_PS3                       "PS3"
 #define VAR_PS4                       "PS4"
 #define VAR_PWD                       "PWD"
 #define VAR_RANDOM                    "RANDOM"
@@ -86,7 +85,8 @@ typedef enum scope_T {
 extern _Bool set_variable(
 	const wchar_t *name, wchar_t *value, scope_T scope, _Bool export)
     __attribute__((nonnull(1)));
-extern _Bool set_array(
+struct variable_T;
+extern struct variable_T *set_array(
 	const wchar_t *name, size_t count, void **values, scope_T scope)
     __attribute__((nonnull));
 extern _Bool set_array_element(
@@ -121,14 +121,13 @@ extern _Bool define_function(const wchar_t *name, struct command_T *body)
 extern struct command_T *get_function(const wchar_t *name)
     __attribute__((nonnull));
 
-extern size_t get_dirstack_size(void);
-extern _Bool parse_dirstack_index(const wchar_t *indexstr,
-	size_t *indexp, const wchar_t **entryp, _Bool printerror)
+extern _Bool parse_dirstack_index(
+	const wchar_t *restrict indexstr, size_t *restrict indexp,
+	const wchar_t **restrict entryp, _Bool printerror)
     __attribute__((nonnull));
-extern _Bool push_dirstack(wchar_t *value)
-    __attribute__((nonnull));
-extern wchar_t *pop_dirstack(void);
-extern _Bool remove_dirstack_entry(size_t index);
+
+extern const struct xoption *const cd_options;
+extern const struct xoption *const pwd_options;
 
 extern int typeset_builtin(int argc, void **argv)
     __attribute__((nonnull));
@@ -142,10 +141,14 @@ extern int getopts_builtin(int argc, void **argv)
     __attribute__((nonnull));
 extern int read_builtin(int argc, void **argv)
     __attribute__((nonnull));
+extern int pushd_builtin(int argc, void **argv)
+    __attribute__((nonnull));
+extern int popd_builtin(int argc, void **argv)
+    __attribute__((nonnull));
 extern int dirs_builtin(int argc, void **argv)
     __attribute__((nonnull));
 extern const char typeset_help[], array_help[], unset_help[], shift_help[],
-       getopts_help[], read_help[], dirs_help[];
+       getopts_help[], read_help[], pushd_help[], popd_help[], dirs_help[];
 
 
 #endif /* YASH_VARIABLE_H */
