@@ -33,12 +33,10 @@ extern void exit_shell_with_status(int status)
 
 struct parseinfo_T;
 
-extern void exec_mbs(const char *code, const char *name, _Bool finally_exit)
-    __attribute__((nonnull(1)));
 extern void exec_wcs(const wchar_t *code, const char *name, _Bool finally_exit)
     __attribute__((nonnull(1)));
 extern void exec_input(FILE *f, const char *name,
-	_Bool intrinput, _Bool finally_exit)
+	_Bool intrinput, _Bool enable_alias, _Bool finally_exit)
     __attribute__((nonnull(1)));
 extern void parse_and_exec(struct parseinfo_T *pinfo, _Bool finally_exit)
     __attribute__((nonnull(1)));
@@ -53,8 +51,8 @@ extern int suspend_builtin(int argc, void **argv)
 extern const char exit_help[], suspend_help[];
 
 
-/* Exits the shell with the exit status of `laststatus'.
- * This function executes EXIT trap and calls `reset_own_pgrp'.
+/* Exits the shell with the current exit status.
+ * This function executes the EXIT trap.
  * This function never returns.
  * This function is reentrant and exits immediately if reentered. */
 void exit_shell(void)
